@@ -80,7 +80,7 @@ def convert_dsa_problem(problem_folder: Path = typer.Argument(".", help="Problem
                                                          "default to problem_folder"),
                         overwrite: bool = typer.Option(False, "--overwrite", "-F",
                                                        help='Force overwriting existing folder'),
-                        testcases_only: bool = typer.Option(False, "--testcases_only", "-T",
+                        testcases_only: bool = typer.Option(True, "--testcases_only", "-T",
                                                             help='Force overwriting existing folder')
                         ):
     if not output:
