@@ -21,7 +21,7 @@ def join_lines(lines):
 
 def make_slug(s):
     s = unidecode(s).lower()
-    print("unidecode: ", s)
+    # print("unidecode: ", s)
     s2 = ""
     for c in s:
         if c.isalnum():
