@@ -1087,11 +1087,11 @@ class ProblemService:
 
     @staticmethod
     def load_themis_testcases(problem_folder, problem: Problem):
-        problem.testcases = TestcaseService.load_testcases(problem_folder, format='themis')
+        problem.testcases = TestcaseService.load_testcases(problem_folder)
 
     @staticmethod
     def load_cms_testcases(problem_folder, problem: Problem):
-        problem.testcases = TestcaseService.load_testcases(os.path.join(problem_folder, "testcases.zip"), format='cms')
+        problem.testcases = TestcaseService.load_testcases(os.path.join(problem_folder, "testcases.zip"))
 
     @staticmethod
     def join_testcases(problem: Problem):
