@@ -13,6 +13,13 @@ from ucode.models.problem import Problem, TestCase
 
 
 class TestcaseService:
+    @staticmethod
+    def remove_carriage_return(testcases):
+        for testcase in testcases:
+            testcase.input = testcase.input.replace("\r\n", "\n")
+            testcase.output = testcase.output.replace("\r\n", "\n")
+        return testcases
+
     @staticmethod
     def load_testcases(path: str):
         """
@@ -32,27 +39,33 @@ class TestcaseService:
 
         if os.path.isfile(path) and not is_zipfile(path):
             CLog.info(f"Loading testcases from single file (ucode format): {path}")
-            return TestcaseService.read_ucode_testcases_from_file(path)
+            return TestcaseService.remove_carriage_return(TestcaseService.read_ucode_testcases_from_file(path))
 
         if is_zipfile(path):
             CLog.info(f"Loading testcases from zipfile: {path}")
             with ZipFile(path, 'r') as myzip:
-                format, testcase_files = TestcaseService.get_testcase_files_list(path, zip_file=myzip)
+                format, testcase_files = TestcaseService.get_testcase_files_list_from_zip(path, zip_file=myzip)
+                CLog.info(f"Detected testcase format `{format}` in zip file: `{path}`")
                 if format == "ucode":
                     inside_archive_file = testcase_files[0]
                     CLog.info(f"Loading testcases (ucode format) from file: {path}\\{inside_archive_file}")
-                    return TestcaseService.read_ucode_testcases_from_file(path, inside_archive_file=inside_archive_file)
+                    return TestcaseService.remove_carriage_return(TestcaseService.read_ucode_testcases_from_file(
+                        path, inside_archive_file=inside_archive_file))
                 else: # format == "themis" / "cms" / "themis" / "hackerrank"
-                    return TestcaseService.read_testcases_from_zip_file(path, inside_archive_files=testcase_files)
+                    return TestcaseService.remove_carriage_return(TestcaseService.read_testcases_from_zip_file(
+                        path, inside_archive_files=testcase_files))
 
         else: # test folder
-            format, testcase_files = TestcaseService.get_testcase_files_list(path, zip_file=None)
+            format, testcase_files = TestcaseService.get_testcase_files_list_from_folder(path)
+            CLog.info(f"Detected testcase format `{format}` in folder: `{path}`")
             if format == "ucode":
                 test_file = testcase_files[0]
                 CLog.info(f"Loading testcases (ucode format) from file: {test_file}")
-                return TestcaseService.read_ucode_testcases_from_file(test_file)
+                return TestcaseService.remove_carriage_return(TestcaseService.read_ucode_testcases_from_file(test_file))
             elif format == "themis":
-                return TestcaseService.read_themis_testcases_from_folder(path)
+                return TestcaseService.remove_carriage_return(TestcaseService.read_themis_testcases_from_folder(path))
+            elif format == "hackerrank":
+                pass
 
         # if format == 'ucode':
         #     return TestcaseService.read_ucode_testcases_from_file(path)
@@ -64,100 +77,132 @@ class TestcaseService:
         #     CLog.error("Testcase format not supported: " + format)
 
     @staticmethod
-    def get_testcase_files_list(path, zip_file=None):
-        if zip_file:
-            all_files = zip_file.namelist()
-            has_input = has_output = False
-            has_folder = False
-            for file in all_files:
-                if file.startswith("input/"):
-                    has_input = True
-                if file.startswith("output/"):
-                    has_output = True
-                if "/" in file:
-                    has_folder = True
-            if has_input and has_output:
-                format = "hackerrank"
-                inputs = [f for f in all_files if f.startswith("input/")]
-                test_files = []
-                for input_file in inputs:
-                    output_file = input_file.replace("input", "output").replace(".in", ".out")
-                    test_files.append((input_file, output_file))
-                print(test_files)
-                return format, test_files
-            elif has_folder:
-                format = "themis"
-                test_files = []
-                inputs = [f for f in all_files if f.endswith(".inp") or f.endswith(".INP")
-                          or f.endswith(".in")  or f.endswith(".IN")]
-                for input_file in inputs:
-                    if input_file.endswith(".INP"):
-                        output_file = input_file[:-4] + ".OUT"
-                    elif input_file.endswith(".inp"):
-                        output_file = input_file[:-4] + ".out"
-                    elif input_file.endswith(".in"):
-                        output_file = input_file[:-3] + ".out"
-                    else:
-                        output_file = input_file[:-3] + ".OUT"
-                    test_files.append((input_file, output_file))
-                print(test_files)
-                return format, test_files
-            else:
-                format = "cms"
-                # format 1: input.xxx & output.xxx
-                # format 2: x.in & x.ou
-                # format 3: x.in & x.ok
-                # format 4: x.in & x.ans
-                # format 5: inputxx.inp & inputxx.out
-                # format 6: x.inp & x.out
-                # format 7: inputxx.txt & inputxx.txt
-                test_files = []
-                input_files = [f for f in all_files if f.endswith(".inp")]
-                if not input_files:
-                    input_files = [f for f in all_files if f.endswith(".in")]
-                if not input_files:
-                    input_files = [f for f in all_files if f.startswith("input")]
-                input_files.sort(key=lambda s: int("".join([c for c in s if "0" <= c <= "9"])))
-                # print("input files", input_files)
-                for input_file in input_files:
-                    # print("input files", input_file)
-                    if input_file.endswith(".inp"):
-                        output_file = input_file.replace(".inp", ".out")
-                    elif input_file.startswith("input"):
-                        output_file = input_file.replace("input", "output")
-                        if output_file not in all_files:
-                            output_file = input_file.replace("input", "ouput")
-                    else:
-                        output_file = input_file.replace(".in", ".ou")
-                        if output_file not in all_files:
-                            output_file = input_file.replace(".in", ".ok")
-                        if output_file not in all_files:
-                            output_file = input_file.replace(".in", ".ans")
-                        if output_file not in all_files:
-                            output_file = input_file.replace(".in", ".out")
-                    # CLog.info(f"Reading a testcase in {input_file}, {output_file}")
-                    test_files.append((input_file, output_file))
-                print(test_files)
-                return format, test_files
+    def find_name_from_list_ignore_case(filename, name_list):
+        for name in name_list:
+            if name.lower() == filename.lower():
+                return name
+        return None
+
+    @staticmethod
+    def get_testcase_files_list_from_folder(path):
+        all_files = glob.glob(os.path.join(path, "*"))
+        folders = [f for f in all_files if os.path.isdir(f)]
+        print("Folder", folders)
+        has_input = has_output = False
+        has_folder = len(folders) > 0
+        for folder in folders:
+            if folder.endswith("input"):
+                has_input = True
+            if folder.endswith("input"):
+                has_output = True
+        if has_input and has_output:
+            format = "hackerrank"
+            CLog.error(f"Reading hackerrank testcases from folder is not supported yet")
+        elif has_folder:
+            format = "themis"
         else:
-            all_files = glob.glob(os.path.join(path, "*"))
-            folders = [f for f in all_files if os.path.isdir(f)]
-            print("Folder", folders)
-            has_input = has_output = False
-            has_folder = len(folders) > 0
-            for folder in folders:
-                if folder.endswith("input"):
-                    has_input = True
-                if folder.endswith("input"):
-                    has_output = True
-            if has_input and has_output:
-                format = "hackerrank"
-                CLog.error(f"Reading hackerrank testcases from folder is not supported yet")
-            elif has_folder:
-                format = "themis"
-            else:
-                format = "cms"
-                CLog.error(f"Reading CMS testcases from folder is not supported yet")
+            format = "cms"
+            CLog.error(f"Reading CMS testcases from folder is not supported yet")
+
+        if not all_files:
+            CLog.error(f"No files found in {path}")
+            return None
+
+        if len(all_files) < 2 or "testcases.txt" in all_files:
+            format = "ucode"
+            return format, all_files
+
+        print(all_files)
+
+        return format, all_files
+
+    @staticmethod
+    def get_testcase_files_list_from_zip(path, zip_file):
+        all_files = zip_file.namelist()
+        has_input = has_output = False
+        folder_count = 0
+        # prefix =
+        # print("all_files", *all_files, sep="\n")
+        for file in all_files:
+            if file.endswith("/"):
+                folder_count += 1
+            first_slash = file.find("/")
+            if file.lower().startswith("input/") \
+                    or first_slash > 0 and file.lower()[first_slash+1:].startswith("input/"):
+                has_input = True
+            if file.lower().startswith("output/") \
+                    or first_slash > 0 and file.lower()[first_slash + 1:].startswith("output/"):
+                has_output = True
+        if has_input and has_output:
+            format = "hackerrank"
+            inputs = [f for f in all_files if (f.startswith("input/") or "/input/" in f) and not f.endswith("/")]
+            test_files = []
+            for input_file in inputs:
+                output_file = input_file.replace("input", "output").replace(".in", ".out")
+                test_files.append((input_file, output_file))
+            print(test_files)
+            return format, test_files
+        elif folder_count > 1:
+            format = "themis"
+            test_files = []
+            inputs = [f for f in all_files if f.lower().endswith(".inp") or f.lower().endswith(".in")]
+            for input_file in inputs:
+                if input_file.lower().endswith(".inp"):
+                    output_file = TestcaseService\
+                        .find_name_from_list_ignore_case(input_file.lower().replace(".inp", ".out"), all_files)
+                elif input_file.lower().endswith(".in"):
+                    output_file = TestcaseService \
+                        .find_name_from_list_ignore_case(input_file.lower().replace(".in", ".out"), all_files)
+                    if not output_file:
+                        output_file = TestcaseService \
+                            .find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ou"), all_files)
+
+                test_files.append((input_file, output_file))
+            print(test_files)
+            return format, test_files
+        else:
+            format = "cms"
+            # format 1: input.xxx & output.xxx
+            # format 2: x.in & x.ou
+            # format 3: x.in & x.ok
+            # format 4: x.in & x.ans
+            # format 5: inputxx.inp & inputxx.out
+            # format 6: x.inp & x.out
+            # format 7: inputxx.txt & inputxx.txt
+            test_files = []
+            input_files = [f for f in all_files if f.lower().endswith(".inp")]
+            if not input_files:
+                input_files = [f for f in all_files if f.lower().endswith(".in")]
+            if not input_files:
+                input_files = [f for f in all_files if f.lower().startswith("input") or "/input" in f.lower()]
+            input_files.sort(key=lambda s: int("".join([c for c in s if "0" <= c <= "9"])))
+            # print("input files", input_files)
+            for input_file in input_files:
+                if input_file.lower().endswith(".inp"):
+                    output_file = TestcaseService.\
+                        find_name_from_list_ignore_case(input_file.replace(".inp", ".out"), all_files)
+                elif input_file.lower().startswith("input") or "/input" in input_file.lower():
+                    output_file = TestcaseService.\
+                        find_name_from_list_ignore_case(input_file.lower().replace("input", "output"), all_files)
+                    if not output_file:
+                        output_file = TestcaseService. \
+                            find_name_from_list_ignore_case(input_file.lower().replace("input", "ouput"), all_files)
+                else:
+                    output_file = TestcaseService. \
+                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ou"), all_files)
+                    if not output_file:
+                        output_file = TestcaseService. \
+                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ok"), all_files)
+                    if not output_file:
+                        output_file = TestcaseService. \
+                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ans"), all_files)
+                    if not output_file:
+                        output_file = TestcaseService. \
+                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".out"), all_files)
+                # CLog.info(f"Reading a testcase in {input_file}, {output_file}")
+                test_files.append((input_file, output_file))
+            print(test_files)
+            return format, test_files
 
         if not all_files:
             CLog.error(f"No files found in {path}")
@@ -174,7 +219,7 @@ class TestcaseService:
     @staticmethod
     def read_testcases_from_zip_file(zipfile, inside_archive_files):
         testcases = []
-        CLog.info(f"Loading CMS testcases from .zip file `{os.path.abspath(zipfile)}`")
+        CLog.info(f"Loading testcases from .zip file `{os.path.abspath(zipfile)}`")
 
         with ZipFile(zipfile, 'r') as myzip:
             for (input_file, output_file) in inside_archive_files:
@@ -440,12 +485,17 @@ if __name__ == "__main__":
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\ucode\testcases.txt"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\ucode\testcases.zip"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\ucode"
+    testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis\sample_testcases.zip"
-    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis"
-    testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\hackerrank\testcases_hackerrank.zip"
+    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis\sample_testcases_nested.zip"
+    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis\sample_testcases_nested_dif_case.zip"
+    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\hackerrank\testcases_hackerrank.zip"
+    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\hackerrank\testcases_hackerrank_nested.zip"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\BANK.zip"
+    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\cms_sample_nested.zip"
     testcases = TestcaseService.load_testcases(testcase_file)
     print(testcases)
     print(len(testcases))
     print(testcases[0])
+    print(testcases[1])
 
