@@ -547,7 +547,8 @@ def create_problems_to_lesson_ucode(env="dev", domain="ucode.vn"):
     # base_folder = "D:\\projects\\freecontest\\formatted\\Free Contest 97"
 
     # base_folder = r"D:\projects\ucode\weekly-algorithm-problems\competitions\teko-hcm"
-    base_folder = r"D:\projects\freecontest\mock_tests\mock01"
+    # base_folder = r"D:\projects\freecontest\mock_tests\mock01"
+    base_folder = r"D:\projects\ucode\competitive_programing_topics\tests\sample_tests\thcs_testTHT"
     # base_folder = "D:\\projects\\ucode\\competitive_programing_topics\\Data_Structures\\Heap_PriorityQueue"
     problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
     # problems = ProblemService.read_all_problems_v1(base_folder, nested_folder=False)
@@ -555,7 +556,7 @@ def create_problems_to_lesson_ucode(env="dev", domain="ucode.vn"):
     # prob_folder = "D:\\projects\\freecontest\\formatted\\Free Contest 80\\groups"
     # problems = [(prob_folder, ProblemService.load(prob_folder))]
 
-    lesson_id = 14395
+    lesson_id = 16614
     ucode = UCode(env=env, domain=domain)
     #
 
@@ -589,7 +590,7 @@ def test_login():
     # ucode = UCode(env="dev", domain="teko.ugrow.vn")
     ucode = UCode(env="prod", domain="ucode.vn")
     # ucode.login("teko@ugrow.vn", "123456")
-    ucode.login("gthuc.nguyen@gmail.com", "ngoaho85")
+    # ucode.login("gthuc.nguyen@gmail.com", "ngoaho85")
     # ucode.login("gthuc.nguyen@gmail.com", "12345678")
     ucode.get_logged_in_user()
 
